@@ -35,6 +35,7 @@
 #include "seahorse-agent.h"
 #include "seahorse-check-button-control.h"
 #include "seahorse-libdialogs.h"
+#include "seahorse-passphrase.h"
 
 #define HIG_SMALL      6        /* gnome hig small space in pixels */
 #define HIG_LARGE     12        /* gnome hig large space in pixels */
@@ -135,7 +136,8 @@ passphrase_response (GtkDialog *dialog, gint response, SeahorseAgentPassReq *pr)
     switch (response) {
     case GTK_RESPONSE_ACCEPT:
         s = seahorse_passphrase_prompt_get (dialog);
-        seahorse_agent_cache_set (pr->id, s != NULL ? s : "", TRUE, TRUE);
+        gtk_widget_hide (GTK_WIDGET (dialog));
+        seahorse_agent_cache_set (pr->id, s != NULL ? s : "", TRUE);
         prompt_done_dialog (pr, TRUE);
         break;
     default:
@@ -151,8 +153,8 @@ seahorse_agent_prompt_pass (SeahorseAgentPassReq *pr)
 
     g_assert (!seahorse_agent_prompt_have ());
     
-    dialog = seahorse_passphrase_prompt_show (NULL, pr->description, 
-                                              pr->prompt, pr->errmsg);
+    dialog = seahorse_passphrase_prompt_show (NULL, pr->errmsg ? pr->errmsg : pr->description, 
+                                              pr->prompt, NULL, FALSE);
     g_signal_connect (dialog, "response", G_CALLBACK (passphrase_response), pr);
     g_current_win = GTK_WIDGET (dialog);
 }
@@ -251,8 +253,8 @@ create_auth_window (SeahorseAgentPassReq *pr)
     gtk_label_set_line_wrap (GTK_LABEL (w), TRUE);
     gtk_box_pack_start (GTK_BOX (box), w, TRUE, FALSE, 0);
     
-    w = seahorse_check_button_control_new(_("Always ask me before using a cached passphrase"),
-                                          SETTING_AUTH);
+    w = gtk_check_button_new_with_mnemonic (_("Always ask me before using a cached passphrase"));
+    seahorse_check_button_gconf_attach (GTK_CHECK_BUTTON (w), SETTING_AUTH);
     
     gtk_box_pack_start (GTK_BOX (box), w, TRUE, FALSE, 0);
                                   
