@@ -65,6 +65,7 @@ enum {
 enum {
     PROP_0,
     PROP_KEY_TYPE,
+    PROP_KEY_DESC,
     PROP_LOCATION
 };
 
@@ -127,6 +128,49 @@ static SeahorseLoadOperation*   seahorse_load_operation_start   (SeahorsePGPSour
 
 static gboolean                 scheduled_dummy                 (gpointer data);
 
+/* -----------------------------------------------------------------------------
+ * EXPORT 
+ */
+ 
+typedef struct _ExportContext {
+    GArray *keyids;
+    guint at;
+    gpgme_data_t data;
+} ExportContext;
+
+static void
+free_export_context (gpointer p)
+{
+    ExportContext *ctx = (ExportContext*)p;
+    if (!ctx)
+        return;
+    g_array_free (ctx->keyids, TRUE);
+    g_free (ctx);
+}
+
+static void
+export_key_callback (SeahorsePGPOperation *pop, ExportContext *ctx)
+{
+    gpgme_error_t gerr;
+    
+    if (seahorse_operation_is_running (SEAHORSE_OPERATION (pop)))
+        seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (pop), NULL, 
+                                               ctx->at, ctx->keyids->len);
+    
+    /* Done, no need to do anything further */
+    if (ctx->at >= ctx->keyids->len)
+        return;
+    
+    /* Do the next key in the list */
+    gerr = gpgme_op_export_start (pop->gctx, g_array_index (ctx->keyids, const char*, ctx->at), 
+                                  0, ctx->data);
+    ctx->at++;
+    
+    if (!GPG_IS_OK (gerr))
+        seahorse_pgp_operation_mark_failed (pop, gerr);
+}
+
+
 /* -----------------------------------------------------------------------------
  * PGP Source
  */
@@ -201,6 +245,10 @@ seahorse_pgp_source_class_init (SeahorsePGPSourceClass *klass)
     g_object_class_install_property (gobject_class, PROP_KEY_TYPE,
         g_param_spec_uint ("key-type", "Key Type", "Key type that originates from this key source.", 
                            0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
+                           
+    g_object_class_install_property (gobject_class, PROP_KEY_DESC,
+        g_param_spec_string ("key-desc", "Key Desc", "Description for keys that originate here.",
+                             NULL, G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class, PROP_LOCATION,
         g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseKeyLoc", 
@@ -319,6 +367,9 @@ seahorse_pgp_source_get_property (GObject *object, guint prop_id, GValue *value,
     case PROP_KEY_TYPE:
         g_value_set_uint (value, SKEY_PGP);
         break;
+    case PROP_KEY_DESC:
+        g_value_set_string (value, _("PGP Key"));
+        break;
     case PROP_LOCATION:
         g_value_set_uint (value, SKEY_LOC_LOCAL);
         break;
@@ -948,9 +999,8 @@ seahorse_pgp_source_export (SeahorseKeySource *sksrc, GList *keys,
     SeahorsePGPSource *psrc;
     SeahorsePGPKey *pkey;
     SeahorseKey *skey;
+    ExportContext *ctx;
     gpgme_error_t gerr = GPG_OK;
-    GArray *akeyids; 
-    gchar **keyids;
     const gchar *keyid;
     GList *l;
     
@@ -975,8 +1025,12 @@ seahorse_pgp_source_export (SeahorseKeySource *sksrc, GList *keys,
     gpgme_set_armor (pop->gctx, TRUE);
     gpgme_set_textmode (pop->gctx, TRUE);
 
-    /* Build list of keyids for export */
-    akeyids = g_array_new (TRUE, TRUE, sizeof (gchar*));
+    /* Export context for asynchronous export */
+    ctx = g_new0(ExportContext, 1);
+    ctx->keyids = g_array_new (TRUE, TRUE, sizeof (gchar*));
+    ctx->at = 0;
+    ctx->data = data;
+    g_object_set_data_full (G_OBJECT (pop), "export-context", ctx, free_export_context);
     
     for (l = keys; l != NULL; l = g_list_next (l)) {
         
@@ -988,9 +1042,9 @@ seahorse_pgp_source_export (SeahorseKeySource *sksrc, GList *keys,
         
         /* Building list */
         keyid = seahorse_pgp_key_get_id (pkey->pubkey, 0);
-        g_array_append_val (akeyids, keyid);
+        g_array_append_val (ctx->keyids, keyid);
         
-        /* TODO: Exporting of secret keys is synchronous */
+        /* Exporting of secret keys is synchronous */
         if (complete && skey->etype == SKEY_PRIVATE) {
             
             gerr = gpgmex_op_export_secret (pop->gctx, 
@@ -1000,16 +1054,15 @@ seahorse_pgp_source_export (SeahorseKeySource *sksrc, GList *keys,
                 break;
         }
     }
-    
-    keyids = (gchar**)g_array_free (akeyids, FALSE);
-    
-    if (GPG_IS_OK (gerr))
-        gerr = gpgme_op_export_ext_start (pop->gctx, (const char**)keyids, 0, data);
-    
+
     if (!GPG_IS_OK (gerr))
         seahorse_pgp_operation_mark_failed (pop, gerr);
+    else
+    {
+        g_signal_connect (pop, "results", G_CALLBACK (export_key_callback), ctx);
+        export_key_callback (pop, ctx);
+    }
     
-    g_free (keyids);
     return SEAHORSE_OPERATION (pop);
 }
 
