@@ -55,6 +55,7 @@
 enum {
     PROP_0,
     PROP_KEY_TYPE,
+    PROP_KEY_DESC,
     PROP_LOCATION,
     PROP_BASE_DIRECTORY
 };
@@ -262,7 +263,7 @@ ssh_key_from_data (SeahorseSSHSource *ssrc, LoadContext *ctx,
     }
 
     /* Make sure it's valid */
-    keyid = seahorse_ssh_key_get_cannonical_id (keydata->keyid);
+    keyid = seahorse_ssh_key_get_cannonical_id (keydata->fingerprint);
     g_return_val_if_fail (keyid, NULL);
 
     /* Does this key exist in the context? */
@@ -500,7 +501,7 @@ seahorse_ssh_source_load (SeahorseKeySource *sksrc, SeahorseKeySourceLoad mode,
         if (filename == NULL)
             break;
 
-        ctx.privfile = g_strconcat (ssrc->priv->ssh_homedir, filename, NULL);
+        ctx.privfile = g_build_filename (ssrc->priv->ssh_homedir, filename, NULL);
         ctx.pubfile = g_strconcat (ctx.privfile, ".pub", NULL);
         
         /* possibly an SSH key? */
@@ -591,7 +592,7 @@ seahorse_ssh_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     ImportContext ctx;
     gchar *contents;
 
-    contents = seahorse_util_write_data_to_text (data, FALSE);
+    contents = seahorse_util_write_data_to_text (data, NULL);
     
     memset (&ctx, 0, sizeof (ctx));
     ctx.ssrc = ssrc;
@@ -611,7 +612,7 @@ seahorse_ssh_source_export (SeahorseKeySource *sksrc, GList *keys,
 {
     SeahorseSSHKeyData *keydata;
     gchar *results = NULL;
-    gchar *prefix = NULL;
+    gchar *raw = NULL;
     GError *error = NULL;
     SeahorseKey *skey;
     GList *l;
@@ -624,7 +625,7 @@ seahorse_ssh_source_export (SeahorseKeySource *sksrc, GList *keys,
         g_assert (SEAHORSE_IS_SSH_KEY (skey));
         
         results = NULL;
-        prefix = NULL;
+        raw = NULL;
         
         keydata = NULL;
         g_object_get (skey, "key-data", &keydata, NULL);
@@ -633,17 +634,20 @@ seahorse_ssh_source_export (SeahorseKeySource *sksrc, GList *keys,
         /* Complete key means the private key */
         if (complete && keydata->privfile) {
             
-            /* Add the seahorse specific prefix */
-            prefix = g_strdup_printf ("%s %s\n", SSH_KEY_SECRET_SIG, keydata->comment ? keydata->comment : "");
-            
             /* And then the data itself */
-            if (!g_file_get_contents (keydata->privfile, &results, NULL, &error))
-                results = NULL;
+            if (!g_file_get_contents (keydata->privfile, &raw, NULL, &error)) {
+                raw = results = NULL;
+            } else {
+                /* Add the seahorse specific prefix */
+                results = g_strdup_printf ("%s %s\n%s\n", SSH_KEY_SECRET_SIG, 
+                                           keydata->comment ? keydata->comment : "",
+                                           raw);
+            }
            
         /* Public key. We should already have the data loaded */
         } else if (keydata->pubfile) { 
             g_assert (keydata->rawdata);
-            results = g_strdup (keydata->rawdata);
+            results = g_strdup_printf ("%s\n", keydata->rawdata);
             
         /* Public key without identity.pub. Export it. */
         } else if (!keydata->pubfile) {
@@ -658,15 +662,15 @@ seahorse_ssh_source_export (SeahorseKeySource *sksrc, GList *keys,
         if (results) {
             
             /* Write the data out */
-            if ((prefix && !write_gpgme_data (data, prefix)) || 
-                !write_gpgme_data (data, results)) {
+            if (write_gpgme_data (data, results)) {
                 g_set_error (&error, G_FILE_ERROR, g_file_error_from_errno (errno),
                              strerror (errno));
             }
             
             g_free (results);
-            g_free (prefix);
         }
+
+        g_free (raw);
         
         if (error != NULL)
             break;
@@ -753,6 +757,9 @@ seahorse_ssh_source_get_property (GObject *object, guint prop_id, GValue *value,
     case PROP_KEY_TYPE:
         g_value_set_uint (value, SKEY_SSH);
         break;
+    case PROP_KEY_DESC:
+        g_value_set_string (value, _("SSH Key"));
+        break;
     case PROP_LOCATION:
         g_value_set_uint (value, SKEY_LOC_LOCAL);
         break;
@@ -807,16 +814,23 @@ seahorse_ssh_source_init (SeahorseSSHSource *ssrc)
     ssrc->priv->monitor_handle = NULL;
 
     ssrc->priv->ssh_homedir = g_strdup_printf ("%s/.ssh/", g_get_home_dir ());
+    
+    /* Make the .ssh directory if it doesn't exist */
+    if (!g_file_test (ssrc->priv->ssh_homedir, G_FILE_TEST_EXISTS)) {
+        if (g_mkdir (ssrc->priv->ssh_homedir, 0700) == -1)
+            g_warning ("couldn't create .ssh directory: %s", ssrc->priv->ssh_homedir);
+            return;
+    }
+    
     uri = gnome_vfs_make_uri_canonical (ssrc->priv->ssh_homedir);
     g_return_if_fail (uri != NULL);
-    
     res = gnome_vfs_monitor_add (&(ssrc->priv->monitor_handle), uri, 
                                  GNOME_VFS_MONITOR_DIRECTORY, 
                                  (GnomeVFSMonitorCallback)monitor_ssh_homedir, ssrc);
     g_free (uri);
     
     if (res != GNOME_VFS_OK) {
-        ssrc->priv->monitor_handle = NULL;        
+        ssrc->priv->monitor_handle = NULL;
         g_return_if_reached ();
     }
 }
@@ -845,6 +859,10 @@ seahorse_ssh_source_class_init (SeahorseSSHSourceClass *klass)
         g_param_spec_uint ("key-type", "Key Type", "Key type that originates from this key source.", 
                            0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
 
+    g_object_class_install_property (gobject_class, PROP_KEY_DESC,
+        g_param_spec_string ("key-desc", "Key Desc", "Description for keys that originate here.",
+                             NULL, G_PARAM_READABLE));
+
     g_object_class_install_property (gobject_class, PROP_LOCATION,
         g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseKeyLoc", 
                            0, G_MAXUINT, SKEY_LOC_INVALID, G_PARAM_READABLE));    
@@ -868,7 +886,6 @@ SeahorseSSHKey*
 seahorse_ssh_source_key_for_filename (SeahorseSSHSource *ssrc, const gchar *privfile)
 {
     SeahorseSSHKeyData *data;
-    SeahorseSSHKey *skey;
     GList *keys, *l;
     int i;
     
@@ -880,8 +897,8 @@ seahorse_ssh_source_key_for_filename (SeahorseSSHSource *ssrc, const gchar *priv
         /* Try to find it first */
         keys = seahorse_context_get_keys (SCTX_APP (), SEAHORSE_KEY_SOURCE (ssrc));
         for (l = keys; l; l = g_list_next (l)) {
-        
-            g_object_get (skey, "key-data", &data, NULL);
+            
+            g_object_get (l->data, "key-data", &data, NULL);
             g_return_val_if_fail (data, NULL);
         
             /* If it's already loaded then just leave it at that */
@@ -903,7 +920,7 @@ gchar*
 seahorse_ssh_source_file_for_algorithm (SeahorseSSHSource *ssrc, guint algo)
 {
     const gchar *pref;
-    gchar *filename;
+    gchar *filename, *t;
     guint i = 0;
     
     switch (algo) {
@@ -922,10 +939,9 @@ seahorse_ssh_source_file_for_algorithm (SeahorseSSHSource *ssrc, guint algo)
     }
     
     for (i = 0; i < ~0; i++) {
-        if (i == 0)
-            filename = g_strdup_printf ("%s/%s", ssrc->priv->ssh_homedir, pref);
-        else
-            filename = g_strdup_printf ("%s/%s.%d", ssrc->priv->ssh_homedir, pref, i);
+        t = (i == 0) ? g_strdup (pref) : g_strdup_printf ("%s.%d", pref, i);
+        filename = g_build_filename (ssrc->priv->ssh_homedir, t, NULL);
+        g_free (t);
         
         if (!g_file_test (filename, G_FILE_TEST_EXISTS))
             break;
@@ -940,6 +956,6 @@ seahorse_ssh_source_file_for_algorithm (SeahorseSSHSource *ssrc, guint algo)
 gchar*
 seahorse_ssh_source_file_for_public (SeahorseSSHSource *ssrc, gboolean authorized)
 {
-    return g_strconcat (ssrc->priv->ssh_homedir, 
+    return g_build_filename (ssrc->priv->ssh_homedir, 
             authorized ? AUTHORIZED_KEYS_FILE : OTHER_KEYS_FILE, NULL);
 }
