@@ -28,6 +28,7 @@
 #include "seahorse-ssh-operation.h"
 #include "seahorse-vfs-data.h"
 #include "seahorse-gpgmex.h"
+#include "seahorse-gtkstock.h"
 
 #define NOTEBOOK "notebook"
 
@@ -43,16 +44,24 @@ do_main (SeahorseWidget *swidget)
     key = SEAHORSE_KEY_WIDGET (swidget)->skey;
     skey = SEAHORSE_SSH_KEY (key);
 
-    widget = glade_xml_get_widget (swidget->xml, "comment-label");
-    if (widget) {
-        text = seahorse_key_get_display_name (key);
-        gtk_label_set_text (GTK_LABEL (widget), text);  
-        g_free (text);
-    }
+    /* Image */
+    widget = seahorse_widget_get_widget (swidget, "key-image");
+    if (widget)
+        gtk_image_set_from_stock (GTK_IMAGE (widget), SEAHORSE_STOCK_KEY_SSH, GTK_ICON_SIZE_DIALOG);
 
-    widget = glade_xml_get_widget (swidget->xml, "fingerprint-label");
+    /* Name and title */
+    text = seahorse_key_get_display_name (key);
+    widget = seahorse_widget_get_widget (swidget, "comment-entry");
+    if (widget)
+        gtk_entry_set_text (GTK_ENTRY (widget), text);
+    widget = seahorse_widget_get_top (swidget);
+    gtk_window_set_title (GTK_WINDOW (widget), text);
+    g_free (text);
+
+    /* Key id */
+    widget = glade_xml_get_widget (swidget->xml, "id-label");
     if (widget) {
-        text = seahorse_key_get_fingerprint (key);  
+        text = seahorse_key_get_display_id (key);
         gtk_label_set_text (GTK_LABEL (widget), text);
         g_free (text);
     }
@@ -72,6 +81,53 @@ do_main (SeahorseWidget *swidget)
                                   seahorse_key_get_trust (key) >= SEAHORSE_VALIDITY_FULL);
 }
 
+static void
+comment_activate (GtkWidget *entry, SeahorseWidget *swidget)
+{
+    SeahorseKey *key;
+    SeahorseSSHKey *skey;
+    SeahorseKeySource *sksrc;
+    SeahorseOperation *op;
+    const gchar *text;
+    gchar *comment;
+    GError *err = NULL;
+    
+    key = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    skey = SEAHORSE_SSH_KEY (key);
+    sksrc = seahorse_key_get_source (key);
+    g_return_if_fail (SEAHORSE_IS_SSH_SOURCE (sksrc));
+    
+    text = gtk_entry_get_text (GTK_ENTRY (entry));
+    
+    /* Make sure not the same */
+    if (skey->keydata->comment && g_utf8_collate (text, skey->keydata->comment) == 0)
+        return;
+
+    gtk_widget_set_sensitive (entry, FALSE);
+    
+    comment = g_strdup (text);
+    op = seahorse_ssh_operation_rename (SEAHORSE_SSH_SOURCE (sksrc), skey, comment);
+    g_free (comment);
+    
+    /* This is usually a quick operation */
+    seahorse_operation_wait (op);
+    
+    if (!seahorse_operation_is_successful (op)) {
+        seahorse_operation_copy_error (op, &err);
+        seahorse_util_handle_error (err, _("Couldn't rename key."));
+        gtk_entry_set_text (GTK_ENTRY (entry), skey->keydata->comment ? skey->keydata->comment : "");
+    }
+    
+    gtk_widget_set_sensitive (entry, TRUE);
+}
+
+static gboolean
+comment_focus_out (GtkWidget* widget, GdkEventFocus *event, SeahorseWidget *swidget)
+{
+    comment_activate (widget, swidget);
+    return FALSE;
+}
+
 static void 
 trust_toggled (GtkToggleButton *button, SeahorseWidget *swidget)
 {
@@ -97,7 +153,7 @@ trust_toggled (GtkToggleButton *button, SeahorseWidget *swidget)
     seahorse_operation_wait (op);
     
     if (!seahorse_operation_is_successful (op)) {
-        seahorse_operation_steal_error (op, &err);
+        seahorse_operation_copy_error (op, &err);
         seahorse_util_handle_error (err, _("Couldn't change authorization for key."));
     }
     
@@ -111,7 +167,7 @@ passphrase_done (SeahorseOperation *op, SeahorseWidget *swidget)
     GtkWidget *w;
 
     if (!seahorse_operation_is_successful (op)) {
-        seahorse_operation_steal_error (op, &err);
+        seahorse_operation_copy_error (op, &err);
         seahorse_util_handle_error (err, _("Couldn't change passhrase for SSH key."));
     }
     
@@ -135,10 +191,7 @@ passphrase_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
     gtk_widget_set_sensitive (w, FALSE);
     
     op = seahorse_ssh_operation_change_passphrase (SEAHORSE_SSH_KEY (skey));
-    if (seahorse_operation_is_running (op))
-        g_signal_connect (op, "done", G_CALLBACK (passphrase_done), swidget);
-    else 
-        passphrase_done (op, swidget);
+    seahorse_operation_watch (op, G_CALLBACK (passphrase_done), NULL, swidget);
 
     /* Running operations ref themselves */
     g_object_unref (op);
@@ -180,7 +233,7 @@ export_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
         gpgmex_data_release (data);
         
         if (!seahorse_operation_is_successful (op))
-            seahorse_operation_steal_error (op, &err);
+            seahorse_operation_copy_error (op, &err);
     }
     
     if (err)
@@ -203,10 +256,11 @@ do_details (SeahorseWidget *swidget)
     key = SEAHORSE_KEY_WIDGET (swidget)->skey;
     skey = SEAHORSE_SSH_KEY (key);
 
-    widget = glade_xml_get_widget (swidget->xml, "id-label");
+    widget = glade_xml_get_widget (swidget->xml, "fingerprint-label");
     if (widget) {
-        label = seahorse_key_get_short_keyid (key); 
-        gtk_label_set_text (GTK_LABEL (widget), label);
+        text = seahorse_key_get_fingerprint (key); 
+        gtk_label_set_text (GTK_LABEL (widget), text);
+        g_free (text);
     }
 
     widget = glade_xml_get_widget (swidget->xml, "algo-label");
@@ -293,6 +347,11 @@ seahorse_ssh_key_properties_new (SeahorseSSHKey *skey)
     
     glade_xml_signal_connect_data (swidget->xml, "trust_toggled", 
                                    G_CALLBACK (trust_toggled), swidget);
+    
+    widget = seahorse_widget_get_widget (swidget, "comment-entry");
+    g_return_if_fail (widget != NULL);
+    g_signal_connect (widget, "activate", G_CALLBACK (comment_activate), swidget);
+    g_signal_connect (widget, "focus-out-event", G_CALLBACK (comment_focus_out), swidget);
 
     if (seahorse_key_get_etype (key) == SKEY_PRIVATE) {
         glade_xml_signal_connect_data (swidget->xml, "export_button_clicked",
